@@ -2,7 +2,10 @@
 #'
 #' Install the source package, figure out the reverse dependencies on CRAN,
 #' download all of their source packages, and run \command{R CMD check} on them
-#' in parallel.
+#' in parallel. The number of parallel downloads can be set via
+#' `options(xfun.rev_check.download_cores = n)` and the number of parallel
+#' checks can be set via `options(mc.cores = n)` (both default to
+#' [parallel::detectCores()]).
 #'
 #' Everything occurs under the current working directory, and you are
 #' recommended to call this function under a designated directory, especially
@@ -20,53 +23,50 @@
 #' failed, and you need to take a look at the log files under that directory.
 #'
 #' The time to finish the check is recorded for each package. As the check goes
-#' on, the total remaining time will be roughly estimated via `n *
-#' mean(times)`, where `n` is the number of packages remaining to be
-#' checked, and `times` is a vector of elapsed time of packages that have
-#' been checked.
+#' on, the total remaining time will be roughly estimated via `n * mean(times)`,
+#' where `n` is the number of packages remaining to be checked, and `times` is a
+#' vector of elapsed time of packages that have been checked.
 #'
 #' If a check on a reverse dependency failed, its \file{*.Rcheck} directory will
 #' be renamed to \file{*.Rcheck2}, and another check will be run against the
-#' CRAN version of the package unless `options(xfun.rev_check.compare =
-#' FALSE)` is set. If the logs of the two checks are the same, it means no new
-#' problems were introduced in the package, and you can probably ignore this
-#' particular reverse dependency. The function `compare_Rcheck()` can be
-#' used to create a summary of all the differences in the check logs under
-#' \file{*.Rcheck} and \file{*.Rcheck2}. This will be done automatically if
+#' CRAN version of the package unless `options(xfun.rev_check.compare = FALSE)`
+#' is set. If the logs of the two checks are the same, it means no new problems
+#' were introduced in the package, and you can probably ignore this particular
+#' reverse dependency. The function `compare_Rcheck()` can be used to create a
+#' summary of all the differences in the check logs under \file{*.Rcheck} and
+#' \file{*.Rcheck2}. This will be done automatically if
 #' `options(xfun.rev_check.summary = TRUE)` has been set.
 #'
-#' A recommended workflow is to use a special directory to run
-#' `rev_check()`, set the global [options()]
-#' `xfun.rev_check.src_dir` and `repos` in the R startup (see
-#' `?`[`Startup`]) profile file `.Rprofile` under this directory,
+#' A recommended workflow is to use a special directory to run `rev_check()`,
+#' set the global [options()] `xfun.rev_check.src_dir` and `repos` in the R
+#' startup (see `?`[`Startup`]) profile file `.Rprofile` under this directory,
 #' and (optionally) set `R_LIBS_USER` in \file{.Renviron} to use a special
 #' library path (so that your usual library will not be cluttered). Then run
 #' `xfun::rev_check(pkg)` once, investigate and fix the problems or (if you
 #' believe it was not your fault) ignore broken packages in the file
-#' \file{00ignore}, and run `xfun::rev_check(pkg)` again to recheck the
-#' failed packages. Repeat this process until all \file{*.Rcheck} directories
-#' are gone.
+#' \file{00ignore}, and run `xfun::rev_check(pkg)` again to recheck the failed
+#' packages. Repeat this process until all \file{*.Rcheck} directories are gone.
 #'
-#' As an example, I set `options(repos = c(CRAN =
-#' 'https://cran.rstudio.com'), xfun.rev_check.src_dir = '~/Dropbox/repo')` in
-#' \file{.Rprofile}, and `R_LIBS_USER=~/R-tmp` in \file{.Renviron}. Then I
-#' can run, for example, `xfun::rev_check('knitr')` repeatedly under a
-#' special directory \file{~/Downloads/revcheck}. Reverse dependencies and their
-#' dependencies will be installed to \file{~/R-tmp}, and \pkg{knitr} will be
-#' installed from \file{~/Dropbox/repo/kintr}.
+#' As an example, I set `options(repos = c(CRAN = 'https://cran.rstudio.com'),
+#' xfun.rev_check.src_dir = '~/Dropbox/repo')` in \file{.Rprofile}, and
+#' `R_LIBS_USER=~/R-tmp` in \file{.Renviron}. Then I can run, for example,
+#' `xfun::rev_check('knitr')` repeatedly under a special directory
+#' \file{~/Downloads/revcheck}. Reverse dependencies and their dependencies will
+#' be installed to \file{~/R-tmp}, and \pkg{knitr} will be installed from
+#' \file{~/Dropbox/repo/kintr}.
 #' @param pkg The package name.
 #' @param which Which types of reverse dependencies to check. See
-#'   [tools::package_dependencies()] for possible values. The
-#'   special value `'hard'` means the hard dependencies, i.e.,
-#'   `c('Depends', 'Imports', 'LinkingTo')`.
+#'   [tools::package_dependencies()] for possible values. The special value
+#'   `'hard'` means the hard dependencies, i.e., `c('Depends', 'Imports',
+#'   'LinkingTo')`.
 #' @param recheck A vector of package names to be (re)checked. If not provided
 #'   and there are any \file{*.Rcheck} directories left by certain packages
-#'   (this often means these packages failed the last time), `recheck` will
-#'   be these packages; if there are no \file{*.Rcheck} directories but a text
-#'   file \file{recheck} exists, `recheck` will be the character vector
-#'   read from this file. This provides a way for you to manually specify the
-#'   packages to be checked. If there are no packages to be rechecked, all
-#'   reverse dependencies will be checked.
+#'   (this often means these packages failed the last time), `recheck` will be
+#'   these packages; if there are no \file{*.Rcheck} directories but a text file
+#'   \file{recheck} exists, `recheck` will be the character vector read from
+#'   this file. This provides a way for you to manually specify the packages to
+#'   be checked. If there are no packages to be rechecked, all reverse
+#'   dependencies will be checked.
 #' @param ignore A vector of package names to be ignored in \command{R CMD
 #'   check}. If this argument is missing and a file \file{00ignore} exists, the
 #'   file will be read as a character vector and passed to this argument.
@@ -79,25 +79,24 @@
 #'   package. The (approximate) total time can be limited by the global option
 #'   `xfun.rev_check.timeout_total`.
 #' @return A named numeric vector with the names being package names of reverse
-#'   dependencies; `0` indicates check success, `1` indicates failure,
-#'   and `2` indicates that a package was not checked due to global
-#'   timeout.
-#' @seealso `devtools::revdep_check()` is more sophisticated, but currently
-#'   has a few major issues that affect me: (1) It always deletes the
+#'   dependencies; `0` indicates check success, `1` indicates failure, and `2`
+#'   indicates that a package was not checked due to global timeout.
+#' @seealso `devtools::revdep_check()` is more sophisticated, but currently has
+#'   a few major issues that affect me: (1) It always deletes the
 #'   \file{*.Rcheck} directories
-#'   (<https://github.com/r-lib/devtools/issues/1395>), which makes it
-#'   difficult to know more information about the failures; (2) It does not
-#'   fully install the source package before checking its reverse dependencies
-#'   (<https://github.com/r-lib/devtools/pull/1397>); (3) I feel it is
-#'   fairly difficult to iterate the check (ignore the successful packages and
-#'   only check the failed packages); by comparison, `xfun::rev_check()`
-#'   only requires you to run a short command repeatedly (failed packages are
+#'   (<https://github.com/r-lib/devtools/issues/1395>), which makes it difficult
+#'   to know more information about the failures; (2) It does not fully install
+#'   the source package before checking its reverse dependencies
+#'   (<https://github.com/r-lib/devtools/pull/1397>); (3) I feel it is fairly
+#'   difficult to iterate the check (ignore the successful packages and only
+#'   check the failed packages); by comparison, `xfun::rev_check()` only
+#'   requires you to run a short command repeatedly (failed packages are
 #'   indicated by the existing \file{*.Rcheck} directories, and automatically
 #'   checked again the next time).
 #'
 #'   `xfun::rev_check()` borrowed a very nice feature from
-#'   `devtools::revdep_check()`: estimating and displaying the remaining
-#'   time. This is particularly useful for packages with huge numbers of reverse
+#'   `devtools::revdep_check()`: estimating and displaying the remaining time.
+#'   This is particularly useful for packages with huge numbers of reverse
 #'   dependencies.
 #' @export
 rev_check = function(
@@ -559,26 +558,26 @@ crandalf_merge = function(pkg) {
   f1
 }
 
+mc_cores = function(name = 'mc.cores') getOption(name, parallel::detectCores())
+
 # mclapply() with a different default for mc.cores and disable prescheduling
 plapply = function(X, FUN, ...) {
-  parallel::mclapply(
-    X, FUN, ..., mc.cores = getOption('mc.cores', parallel::detectCores()),
-    mc.preschedule = FALSE
-  )
+  parallel::mclapply(X, FUN, ..., mc.cores = mc_cores(), mc.preschedule = FALSE)
 }
 
 # download the source package from CRAN
 download_tarball = function(p, db = available.packages(type = 'source'), dir = '.', retry = 3) {
   if (!dir_exists(dir)) dir.create(dir, recursive = TRUE)
   z = file.path(dir, sprintf('%s_%s.tar.gz', p, db[p, 'Version']))
-  mapply(function(p, z) {
+  parallel::mcmapply(function(p, z) {
     # remove other versions of the package tarball
     unlink(setdiff(list.files(dir, sprintf('^%s_.+.tar.gz', p), full.names = TRUE), z))
+    u = paste(db[p, 'Repository'], basename(z), sep = '/')
     for (i in seq_len(retry)) {
       if (file_exists(z)) break
-      try(download.file(paste(db[p, 'Repository'], basename(z), sep = '/'), z, mode = 'wb'))
+      try(download.file(u, z, quiet = TRUE, mode = 'wb'))
     }
-  }, p, z, SIMPLIFY = FALSE)
+  }, p, z, SIMPLIFY = FALSE, mc.cores = mc_cores('xfun.rev_check.download_cores'))
   z
 }
 
@@ -640,12 +639,10 @@ compare_Rcheck = function(status_only = TRUE, output = '00check_diffs.md') {
     )
   }
   if (length(res) == 0) return()
+  res = c('---', 'title: xfun::rev_check() results', '---', '', res)
   xfun::write_utf8(res, output)
-  if (!loadable('markdown')) return(output)
-  markdown::mark_html(
-    text = res,
-    output = html_file <- with_ext(output, 'html')
-  )
+  if (!loadable('litedown')) return(output)
+  html_file = litedown::mark(output, text = res)
   if (!getOption('xfun.rev_check.keep_md', FALSE)) unlink(output)
   html_file
 }
@@ -715,63 +712,3 @@ find_missing_latex = function() {
   if (file_exists(f <- 'latex.txt')) append_unique(pkgs, f)
   pkgs
 }
-
-# run revdepcheck::cloud_check()
-cloud_check = function(pkgs = NULL, batch_size = 200) {
-  call_fun = function(name, ..., verbose = FALSE) {
-    fun = getFromNamespace(name, 'revdepcheck')
-    if (verbose) fun(...) else suppressMessages(fun(...))
-  }
-  tgz = pkg_build()  # tarball
-  pkg = gsub('_.*$', '', tgz)
-  if (length(pkgs) == 0) pkgs = setdiff(call_fun('cran_revdeps', pkg, bioc = TRUE), pkg)
-  N = length(pkgs)
-  jobs = broken = NULL
-  rver = format(getRversion())
-  check = function() {
-    # make sure to check at least 2 packages
-    if (length(pkgs) == 1) pkgs = c(pkgs, if (length(broken)) broken[1] else pkgs)
-    try_check = function() {
-      call_fun('cloud_check', tarball = tgz, r_version = rver, revdep_packages = head(pkgs, batch_size))
-    }
-    jobs <<- c(jobs, tryCatch(
-      try_check(),
-      error = function(e) {
-        if (getRversion() != rver) stop(e)  # already tried a different version
-        # if the current R version doesn't work, use the highest supported version
-        r = ".*?\\[(('([0-9.]+)'(,\\s+)?)+)].*"
-        x = grep(r, e$message, value = TRUE)
-        x = gsub(r, '\\1', x)
-        v = unlist(strsplit(x, "('|,\\s+)"))
-        v = v[v != ''][1]
-        if (length(v) != 1 || is.na(v)) stop(e)
-        rver <<- v
-        try_check()
-      }
-    ))
-    pkgs <<- tail(pkgs, -batch_size)
-    message(N - length(pkgs), '... ', appendLF = FALSE)
-  }
-  # if there are more than batch_size revdeps, submit one batch at one time
-  message('Checking ', N, ' packages: ', appendLF = FALSE)
-  while (length(pkgs) > 0) check()
-  message('All jobs submitted.')
-  for (job in jobs) {
-    call_fun('cloud_status', job, update_interval = 300, verbose = TRUE)
-  }
-  for (job in jobs) {
-    if (length(res <- call_fun('cloud_broken', job))) {
-      call_fun('cloud_report', job)
-      for (p in res) print(call_fun('cloud_details', job, revdep = p, verbose = TRUE))
-      fs = list.files(file.path('revdep/cloud.noindex', job), full.names = TRUE)
-      # only keep results from broken packages
-      unlink(fs[!basename(fs) %in% c(res, paste0(res, '.tar.gz'))], recursive = TRUE)
-      broken = unique(c(res, broken))
-    }
-  }
-  if (length(broken)) {
-    stop('Package(s) broken: ', paste(broken, collapse = ' '), call. = FALSE)
-  } else {
-    message('All reverse dependencies are good!')
-  }
-}
